'use strict';

/*
 * jQuery Attach
 * Simple jQuery plugin to ensure one-time behavior binding
 * (c) 2016 Joshua Cerbito - https://github.com/joshuacerbito/jquery-attach-behavior.git
 * Released under the ISC license
 */

(function ($) {
  'use strict';

  var typeOf = function typeOf(str) {
    return Object.prototype.toString.call(str).replace(/^\[object\s(.*)\]$/, "$1");
  };

  $.fn.attachBehavior = function () {
    var trigger = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';
    var cb = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : function () {};
    var showErrors = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;

    var _this = this;
    var _showErrors = !!showErrors;
    var _jqueryEvent = $._data($(_this)[0], 'events') || {};

    if (_showErrors && _jqueryEvent.hasOwnProperty(trigger)) {
      throw Error('The "' + trigger + '" event has already been assigned to this behavior:\n' + _jqueryEvent[trigger][0].handler);
    }

    if (_showErrors && typeof cb !== 'function') {
      throw TypeError('Expected a "function", instead saw a "' + typeOf(cb) + '": "' + cb + '"');
    }

    _this.on(trigger, function (evt) {
      cb.apply(_this, arguments);
    });

    return _this;
  };

  $.fn.detachBehavior = function (trigger) {
    var _this = this;
    var _jqueryEvent = $._data($(_this)[0], 'events') || {};

    // fix this
    if (!!_jqueryEvent.hasOwnProperty(trigger)) {
      _this.off(trigger);
    }

    return _this;
  };

  $.fn.getBehavior = function () {
    var trigger = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;

    var _binding = $._data($(this)[0], 'events');
    var _return = {};

    if (!!trigger) {
      _return = {
        trigger: _binding[trigger][0].type,
        callback: _binding[trigger][0].handler,
        _binding: _binding
      };
    } else {
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = Object.keys(_binding)[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var key = _step.value;

          _return[key] = {
            trigger: _binding[key][0].type,
            callback: _binding[key][0].handler,
            _binding: _binding
          };
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }
    }

    return _return;
  };
})(jQuery);
