/*
 * jQuery Attach
 * Simple jQuery plugin to ensure one-time behavior binding
 * (c) 2016 Joshua Cerbito - https://github.com/joshuacerbito/jquery-attach-behavior.git
 * Released under the ISC license
 */

(function($) {
  'use strict';

  const typeOf = function (str) {
    return Object.prototype.toString.call(str).replace(/^\[object\s(.*)\]$/, "$1")
  };

  $.fn.attachBehavior = function(trigger = '', cb = function(){}, showErrors = true) {
    let _this = this;
    let _showErrors = !!showErrors;
    let _jqueryEvent = $._data($(_this)[0], 'events') || {};

    if ( _showErrors && _jqueryEvent.hasOwnProperty(trigger) ) {
      throw Error(`The "${trigger}" event has already been assigned to this behavior:\n${_jqueryEvent[trigger][0].handler}`);
    }

    if ( _showErrors && (typeof cb !== 'function') ) {
      throw TypeError(`Expected a "function", instead saw a "${typeOf(cb)}": "${cb}"`);
    }

    _this.on(trigger, function(evt) {
      cb.apply(_this, arguments);
    });

    return _this;
  };

  $.fn.detachBehavior = function(trigger) {
    let _this = this;
    let _jqueryEvent = $._data($(_this)[0], 'events') || {};

    // fix this
    if ( !!_jqueryEvent.hasOwnProperty(trigger) ) {
      _this.off(trigger);
    }

    return _this;
  };

  $.fn.getBehavior = function(trigger = null) {
    let _binding = $._data($(this)[0], 'events');
    let _return = {};

    if (!!trigger) {
      _return = {
        trigger: _binding[trigger][0].type,
        callback: _binding[trigger][0].handler,
        _binding: _binding
      };
    } else {
      for (let key of Object.keys(_binding)) {
        _return[key] = {
          trigger: _binding[key][0].type,
          callback: _binding[key][0].handler,
          _binding: _binding
        };
      }
    }

    return _return;
  };
})(jQuery);
